/**
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 */
package net.amoabeng.util.build;


import java.io.IOException;
import java.nio.charset.Charset;
import java.nio.file.Path;
import java.util.Collection;
import java.util.List;

import static java.nio.file.Files.createDirectories;
import static java.nio.file.Files.write;
import static java.util.Collections.singletonList;
import static net.amoabeng.util.CollectionUtils.split;

/**
 * @author Manuel Amoabeng
 */
class CssResourceBuilder implements ResourceBuilder {

    private static final int MAX_CSS_INCLUDES = 30;

    private final Path targetFile;
    private final Charset charset;
    private final ResourceBuilder resourceBuilder;
    private final LinkBuilder importLinkBuilder;
    private final LinkBuilder linkBuilder;

    CssResourceBuilder(
            Path targetFile,
            Charset charset,
            ResourceBuilder resourceBuilder,
            LinkBuilder importLinkBuilder,
            LinkBuilder linkBuilder) {
        this.targetFile = targetFile;
        this.charset = charset;
        this.resourceBuilder = resourceBuilder;
        this.importLinkBuilder = importLinkBuilder;
        this.linkBuilder = linkBuilder;
    }

    @Override
    public List<Link> build(FileSource fileSource) throws IOException {
        int i = 0;
        Path buildTargetFile = fileSource.getBuildRoot().resolve(targetFile);
        List<Link> allLinks = resourceBuilder.build(fileSource);

        while (allLinks.size() > MAX_CSS_INCLUDES) {
            for (Collection<Link> links : split(allLinks, MAX_CSS_INCLUDES)) {
                String importFileName = (++i) + "_" + buildTargetFile.getFileName();
                Path importFile = buildTargetFile.resolveSibling(importFileName);
                byte[] data = writeImportFile(links, importFile);
                Link importLink = importLinkBuilder.createLink(importFile, data);
                allLinks.add(importLink);
                allLinks.removeAll(links);
            }
        }
        byte[] data = writeImportFile(allLinks, buildTargetFile);
        Link targetLink = linkBuilder.createLink(buildTargetFile, data);
        return singletonList(targetLink);
    }

    private byte[] writeImportFile(Collection<Link> links, Path importFile) throws IOException {
        StringBuilder content = new StringBuilder();
        for (Link link : links) {
            Link relativeLink = link.relativize(importFile);
            content.append(relativeLink).append("\n");
        }
        byte[] data = content.toString().getBytes(charset);
        createDirectories(importFile.getParent());
        write(importFile, data);
        return data;
    }
}
