/**
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 */
package net.amoabeng.util.build;


import net.amoabeng.util.ReflectUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;
import java.nio.charset.Charset;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.*;

import static net.amoabeng.util.CollectionUtils.join;
import static net.amoabeng.util.CollectionUtils.merge;
import static net.amoabeng.util.ReflectUtils.newInstance;
import static net.amoabeng.util.build.Settings.key;



/**
 * <pre>
 * Creates and configures a {@link ResourceManager}.
 *
 * The default settings will make it work on a typical maven webapp project layout like this:
 *
 * [app-name]
 *      src
 *          main
 *              webapp
 *                  js
 *                      ...
 *                  css
 *                      ...
 *      target
 *          [app-name]
 *              js
 *                  all.js
 *              css
 *                  all.css
 *
 * This is the default configuration:
 *
 * baseDir:                         .                  - the working dir is considered to be the project root
 * charset:                         UTF-8              - the encoding of the source files
 * buildRoot:                       target/[app-name]  - the directory the build is located at,
 *                                                       the folder [app-name] is derived from the root directory
 *                                                       of the project
 * mode:                            production         - mode can either be debug or production. in production mode all
 *                                                       all css and js files will be minified and merged into one file
 * sourceRoot:                      src/main/webapp    - the root folder of files to process
 * watch:                           false              - flag determining if changes on source files trigger builds
 *
 * jsBuilder.type:                  builder
 * jsBuilder.producerType:          net.amoabeng.util.build.{@link JsResourceBuilderProducer}
 * jsBuilder.targetFile:            all.js
 * jsBuilder.fileSources:           jsFileSource
 *
 * cssBuilder.type:                 builder
 * cssBuilder.producerType:         net.amoabeng.util.build.{@link CssResourceBuilderProducer}
 * cssBuilder.targetFile:           all.css
 * cssBuilder.directoryTargetFile:  merged.css
 * cssBuilder.fileSources:          cssFileSource
 *
 * templateBuilder.type:            template
 * templateBuilder.producerType:    net.amoabeng.util.build.{@link SimpleTemplateBuilderProducer}
 * templateBuilder.fileSources:     templateFileSource
 *
 * jsFileSource.type:               fileSource
 * jsFileSource.producerType:       net.amoabeng.util.build.{@link PathMatcherFileSourceProducer}
 * jsFileSource.pathMatcher:        **&#47;*.js
 * jsFileSource.sourceRoot:         js
 * jsFileSource.buildRoot:          js
 *
 * cssFileSource.type:              fileSource
 * cssFileSource.producerType:      net.amoabeng.util.build.{@link PathMatcherFileSourceProducer}
 * cssFileSource.pathMatcher:       **&#47;*.css
 * cssFileSource.sourceRoot:        css
 * cssFileSource.buildRoot:         css
 *
 * templateFileSource.type:         fileSource
 * templateFileSource.producerType: net.amoabeng.util.build.{@link PathMatcherFileSourceProducer}
 * templateFileSource.pathMatcher:  **&#47;*.html
 * templateFileSource.sourceRoot:   .
 * templateFileSource.buildRoot:    .
 * </pre>
 *
 * @author Manuel Amoabeng
 */
class DefaultResourceManagerProducer implements ResourceManagerProducer {


    private static final String
            JS_FILES = "jsFileSource",
            CSS_FILES = "cssFileSource",
            TEMPLATE_FILES = "templateFileSource",
            JS_BUILDER = "jsBuilder",
            CSS_BUILDER = "cssBuilder",
            TEMPLATE_BUILDER = "templateBuilder",
            PATH_MATCHER = "pathMatcher",
            TARGET_FILE = "targetFile",
            DIRECTORY_TARGET_FILE = "directoryTargetFile";

    private static final Logger LOG = LoggerFactory.getLogger(DefaultResourceManagerProducer.class);

    @Override
    public ResourceManager newResourceManager(Map<String, String> settingsMap) throws IOException {
        settingsMap = merge(settingsMap, getDefaults());

        LOG.info("Creating ResourceManager with settings:\n{}", join(settingsMap));

        Map<String, ResourceBuilder> resourceBuilders = new HashMap<>();
        Map<String, TemplateBuilder> templateBuilders = new HashMap<>();
        Map<String, FileSource> fileSources = new HashMap<>();
        Set<String> ids = new HashSet<>();
        for (String key : settingsMap.keySet()) {
            int i = key.indexOf(".type");
            if (i > 0) {
                ids.add(key.substring(0, i));
            }
        }
        for (String id : ids) {
            Settings settings = new Settings(id, settingsMap);
            switch (settings.getType()) {
                case ResourceManagerProducer.FILE_SOURCE: {
                    FileSourceProducer producer = (FileSourceProducer) newInstance(settings.get(PRODUCER_TYPE));
                    FileSource fileSource = producer.newFileSource(settings);
                    fileSources.put(id, fileSource);
                }
                break;
                case ResourceManagerProducer.TEMPLATE: {
                    TemplateBuilderProducer producer =
                            (TemplateBuilderProducer) newInstance(settings.get(PRODUCER_TYPE));
                    TemplateBuilder builder = producer.newTemplateBuilder(settings);
                    String[] fileSourceIds = settings.get(FILE_SOURCES).split("\\s,\\s");
                    for (String fileSourceId : fileSourceIds) {
                        templateBuilders.put(fileSourceId, builder);
                    }
                }
                break;
                case ResourceManagerProducer.BUILDER: {
                    ResourceBuilderProducer producer =
                            (ResourceBuilderProducer) ReflectUtils.newInstance(settings.get(PRODUCER_TYPE));
                    ResourceBuilder builder = producer.newResourceBuilder(settings);
                    String[] fileSourceIds = settings.get(FILE_SOURCES).split("\\s,\\s");
                    for (String fileSourceId : fileSourceIds) {
                        resourceBuilders.put(fileSourceId, builder);
                    }
                }
                break;
            }
        }
        boolean buildOnChange = "true".equalsIgnoreCase(settingsMap.get(ResourceManagerProducer.BUILD_ON_CHANGE));
        return new ResourceManager(buildOnChange, resourceBuilders, templateBuilders, fileSources);
    }

    private Map<String, String> getDefaults() {

        Path baseDir = Paths.get("").toAbsolutePath();
        Map<String, String> defaults = new LinkedHashMap<>();

        // global settings
        defaults.put(BASE_DIR, baseDir.toString());
        defaults.put(SOURCE_ROOT, Paths.get("src", "main", "webapp").toString());
        defaults.put(BUILD_ROOT, Paths.get("target").resolve(baseDir.getFileName()).toString());
        defaults.put(CHARSET, Charset.defaultCharset().name());
        defaults.put(MODE, "production");

        // js builder
        defaults.put(key(JS_BUILDER, TYPE), BUILDER);
        defaults.put(key(JS_BUILDER, PRODUCER_TYPE), JsResourceBuilderProducer.class.getName());
        defaults.put(key(JS_BUILDER, TARGET_FILE), "all.js");
        defaults.put(key(JS_BUILDER, FILE_SOURCES), JS_FILES);

        // css builder
        defaults.put(key(CSS_BUILDER, TYPE), BUILDER);
        defaults.put(key(CSS_BUILDER, PRODUCER_TYPE), CssResourceBuilderProducer.class.getName());
        defaults.put(key(CSS_BUILDER, TARGET_FILE), "all.css");
        defaults.put(key(CSS_BUILDER, DIRECTORY_TARGET_FILE), "merged.css");
        defaults.put(key(CSS_BUILDER, FILE_SOURCES), CSS_FILES);

        // template builder
        defaults.put(key(TEMPLATE_BUILDER, TYPE), TEMPLATE);
        defaults.put(key(TEMPLATE_BUILDER, PRODUCER_TYPE), SimpleTemplateBuilderProducer.class.getName());
        defaults.put(key(TEMPLATE_BUILDER, FILE_SOURCES), TEMPLATE_FILES);

        // file sources
        String fileSourceProducerType = PathMatcherFileSourceProducer.class.getName();

        defaults.put(key(JS_FILES, TYPE), FILE_SOURCE);
        defaults.put(key(JS_FILES, PRODUCER_TYPE), fileSourceProducerType);
        defaults.put(key(JS_FILES, PATH_MATCHER), "**/*.js");
        defaults.put(key(JS_FILES, SOURCE_ROOT), "js");
        defaults.put(key(JS_FILES, BUILD_ROOT), "js");

        defaults.put(key(CSS_FILES, TYPE), FILE_SOURCE);
        defaults.put(key(CSS_FILES, PRODUCER_TYPE), fileSourceProducerType);
        defaults.put(key(CSS_FILES, PATH_MATCHER), "**/*.css");
        defaults.put(key(CSS_FILES, SOURCE_ROOT), "css");
        defaults.put(key(CSS_FILES, BUILD_ROOT), "css");

        defaults.put(key(TEMPLATE_FILES, TYPE), FILE_SOURCE);
        defaults.put(key(TEMPLATE_FILES, PRODUCER_TYPE), fileSourceProducerType);
        defaults.put(key(TEMPLATE_FILES, PATH_MATCHER), "**/*.html");
        defaults.put(key(TEMPLATE_FILES, SOURCE_ROOT), "");
        defaults.put(key(TEMPLATE_FILES, BUILD_ROOT), "");

        return defaults;
    }
}
