/**
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 */
package net.amoabeng.util.build;

import com.yahoo.platform.yui.compressor.JavaScriptCompressor;
import net.amoabeng.util.io.FileUtils;
import org.mozilla.javascript.ErrorReporter;
import org.mozilla.javascript.EvaluatorException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.*;
import java.nio.charset.Charset;
import java.nio.file.Path;
import java.nio.file.attribute.FileTime;
import java.util.*;

import static java.nio.file.Files.*;
import static net.amoabeng.util.io.FileUtils.resolve;

/**
 * @author Manuel Amoabeng
 */
class JsResourceBuilder implements ResourceBuilder {

    private static final Logger LOG = LoggerFactory.getLogger(JsResourceBuilder.class);

    private final LinkBuilder linkBuilder;
    private final Charset charset;
    private final Path targetFile;

    private final Map<Path, Link> cachedLinks = new HashMap<>();

    public JsResourceBuilder(LinkBuilder linkBuilder, Charset charset, Path targetFile) {
        this.linkBuilder = linkBuilder;
        this.charset = charset;
        this.targetFile = targetFile;
    }

    @Override
    public List<Link> build(FileSource fileSource) throws IOException {

        Path sourceRoot = fileSource.getSourceRoot();
        Path buildRoot = fileSource.getBuildRoot();
        List<Path> rootPaths = resolve(sourceRoot, fileSource.getPaths());
        Path targetFile = buildRoot.resolve(this.targetFile).normalize();
        Link link = cachedLinks.get(targetFile);

        if (isModified(rootPaths, targetFile, link)) {

            byte[] data = compress(rootPaths);
            createDirectories(targetFile.getParent());
            write(targetFile, data);
            link = linkBuilder.createLink(targetFile, data);
            cachedLinks.put(targetFile, link);
        }

        List<Link> links = new ArrayList<>();
        links.add(link);
        return links;
    }

    private boolean isModified(List<Path> rootPaths, Path targetFile, Link link) throws IOException {
        if (link == null || !exists(targetFile)) {
            return true;
        }
        FileTime lastModified = getLastModifiedTime(targetFile);
        for (Path rootPath : rootPaths) {
            if (lastModified.compareTo(getLastModifiedTime(rootPath)) < 0) {
                return true;
            }
        }
        return false;
    }

    private byte[] compress(Collection<Path> paths) throws IOException {
        try (
                InputStream inputStream = FileUtils.newInputStream(paths);
                Reader reader = new InputStreamReader(inputStream, charset);
                StringWriter writer = new StringWriter()) {

            JavaScriptCompressor compressor = new JavaScriptCompressor(reader, new ErrorReporter() {
                @Override
                public void warning(String detail, String sourceName, int lineNumber, String lineSource, int columnNumber) {
                    LOG.warn(
                            "file: {}, line: {}, column: {}, message: {}, source: {}",
                            new Object[]{sourceName, lineNumber, columnNumber, detail, lineSource});
                }

                @Override
                public void error(String detail, String sourceName, int lineNumber, String lineSource, int columnNumber) {
                    LOG.error(
                            "file: {}, line: {}, column: {}, message: {}, source: {}",
                            new Object[]{sourceName, lineNumber, columnNumber, detail, lineSource});
                }

                @Override
                public EvaluatorException runtimeError(String detail, String sourceName, int lineNumber, String lineSource, int columnNumber) {
                    return new EvaluatorException(detail, sourceName, lineNumber, lineSource, columnNumber);
                }
            });

            int lineBreak = 1000;
            boolean munge = true;
            boolean verbose = false;
            boolean preserveAllSemiColons = false;
            boolean disableOptimizations = false;
            compressor.compress(writer, lineBreak, munge, verbose, preserveAllSemiColons, disableOptimizations);
            String content = writer.toString();
            return content.getBytes(charset);
        }
    }
}
