/**
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 */
package net.amoabeng.util.build;

import java.math.BigInteger;
import java.nio.file.Path;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;

/**
 * Represents a link to a file.
 *
 * @author Manuel Amoabeng
 */
public final class Link {

    private final String hashPrefix;
    private final Path target;
    private final String start;
    private final String end;
    private final String hash;

    /**
     * @param target     The target of the link
     * @param data       The data contained in the target file
     * @param start      The start of the link string
     * @param end        The end of the link string
     * @param hashPrefix The prefix for the hash in the link string
     */
    public Link(Path target, byte[] data, String start, String end, String hashPrefix) {
        this.target = target;
        this.start = start;
        this.end = end;
        this.hashPrefix = hashPrefix;
        this.hash = getMd5Hash(data);
    }

    /**
     * @param target     The target of the link
     * @param hash       The hash of the data contained in the target file
     * @param start      The start of the link string
     * @param end        The end of the link string
     * @param hashPrefix The prefix for the hash in the link string
     */
    public Link(Path target, String hash, String start, String end, String hashPrefix) {
        this.target = target;
        this.start = start;
        this.end = end;
        this.hashPrefix = hashPrefix;
        this.hash = hash;
    }

    /**
     * @param sourceFile The file the link should be set up for
     * @return A link with the target path set up relative to the given sourceFile
     */
    public Link relativize(Path sourceFile) {
        return new Link(sourceFile.getParent().relativize(target), hash, start, end, hashPrefix);
    }

    /**
     * @return The string representation of this link
     */
    @Override
    public String toString() {
        String path = target.toString().replaceAll("\\\\", "/");
        return start + path + hashPrefix + hash + end;
    }

    private String getMd5Hash(byte[] data) {
        MessageDigest messageDigest;
        try {
            messageDigest = MessageDigest.getInstance("MD5");
        } catch (final NoSuchAlgorithmException e) {
            throw new RuntimeException(e); // should never happen
        }
        messageDigest.update(data, 0, data.length);
        return new BigInteger(1, messageDigest.digest()).toString(16);
    }
}
