/**
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 */
package net.amoabeng.util.build;

import com.yahoo.platform.yui.compressor.CssCompressor;
import net.amoabeng.util.CompareUtils;
import net.amoabeng.util.io.FileUtils;

import java.io.*;
import java.nio.charset.Charset;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.nio.file.attribute.FileTime;
import java.util.*;

/**
 * @author Manuel Amoabeng
 */
class MergeCssResourceBuilder implements ResourceBuilder {

    private final LinkBuilder linkBuilder;
    private final Charset charset;
    private final Path directoryTargetFile;

    private final Map<Path, Link> cachedLinks = new HashMap<>();

    public MergeCssResourceBuilder(LinkBuilder linkBuilder, Charset charset, Path directoryTargetFile) {
        this.linkBuilder = linkBuilder;
        this.charset = charset;
        this.directoryTargetFile = directoryTargetFile;
    }

    @Override
    public List<Link> build(FileSource fileSource) throws IOException {

        List<Link> links = new ArrayList<>();
        Path sourceRoot = fileSource.getSourceRoot();
        Path buildRoot = fileSource.getBuildRoot();
        List<Path> rootPaths = fileSource.getPaths();
        Map<Path, List<Path>> filesByDirectory = filesByDirectory(rootPaths);

        for (Map.Entry<Path, List<Path>> entry : filesByDirectory.entrySet()) {
            List<Path> paths = entry.getValue();
            Path targetFile = buildRoot.resolve(entry.getKey()).resolve(directoryTargetFile);
            Link link = cachedLinks.get(targetFile);
            if (isModified(paths, targetFile, link)) {
                byte[] data = compress(FileUtils.resolve(sourceRoot, paths));
                Files.createDirectories(targetFile.getParent());
                Files.write(targetFile, data);
                link = linkBuilder.createLink(targetFile, data);
                cachedLinks.put(targetFile, link);
            }
            links.add(link);
        }
        return links;
    }

    private boolean isModified(List<Path> rootPaths, Path targetFile, Link link) throws IOException {
        if (link == null || !Files.exists(targetFile)) {
            return true;
        }
        FileTime lastModified = Files.getLastModifiedTime(targetFile);
        for (Path rootPath : rootPaths) {
            if (lastModified.compareTo(Files.getLastModifiedTime(rootPath)) < 0) {
                return true;
            }
        }
        return false;
    }

    private Map<Path, List<Path>> filesByDirectory(Collection<Path> paths) {
        Map<Path, List<Path>> map = new LinkedHashMap<>();
        for (Path path : paths) {
            Path parent = CompareUtils.nvl(path.getParent(), Paths.get(""));
            List<Path> filesByDirectory = map.get(parent);
            if (filesByDirectory == null) {
                filesByDirectory = new ArrayList<>();
                map.put(parent, filesByDirectory);
            }
            filesByDirectory.add(path);
        }
        return map;
    }

    private byte[] compress(Collection<Path> paths) throws IOException {
        try (
                InputStream inputStream = FileUtils.newInputStream(paths);
                Reader reader = new InputStreamReader(inputStream, charset);
                StringWriter writer = new StringWriter()) {
            CssCompressor compressor = new CssCompressor(reader);
            int lineBreak = 250;
            compressor.compress(writer, lineBreak);
            String content = writer.toString();
            return content.getBytes(charset);
        }
    }
}
