/**
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 */
package net.amoabeng.util.build;

import net.amoabeng.util.CollectionUtils;
import net.amoabeng.util.io.WatchEventObserver;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.Closeable;
import java.io.IOException;
import java.nio.file.Path;
import java.nio.file.WatchEvent;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;

import static net.amoabeng.util.CollectionUtils.join;
import static net.amoabeng.util.io.FileUtils.newWatchEventObserver;

/**
 * <pre>
 * Three kinds of processors are put to work in {@link ResourceManager}s:
 *
 * {@link FileSource}s provide source files,
 * {@link ResourceBuilder} build web resources and links to them from source files and
 * {@link TemplateBuilder}s include web resources in template files.
 * </pre>
 *
 * @author Manuel Amoabeng
 */
public final class ResourceManager implements Closeable {

    private static final Logger LOG = LoggerFactory.getLogger(ResourceManager.class);


    private final Map<String, ResourceBuilder> resourceBuildersByFileSourceId;
    private final Map<String, TemplateBuilder> templateBuildersByFileSourceId;
    private final Map<String, FileSource> fileSourcesById;

    private WatchEventObserver watchEventObserver;

    ResourceManager(
            boolean buildOnChange,
            Map<String, ResourceBuilder> resourceBuildersByFileSourceId,
            Map<String, TemplateBuilder> templateBuildersByFileSourceId,
            Map<String, FileSource> fileSourcesById) throws IOException {

        this.resourceBuildersByFileSourceId = resourceBuildersByFileSourceId;
        this.templateBuildersByFileSourceId = templateBuildersByFileSourceId;
        this.fileSourcesById = fileSourcesById;


        if (buildOnChange) {
            watchEventObserver = newWatchEventObserver();
            for (FileSource fileSource : fileSourcesById.values()) {
                watchEventObserver.register(fileSource.getSourceRoot());
            }
            watchEventObserver.addListener(3, TimeUnit.SECONDS, new WatchEventObserver.DelayedListener() {
                @Override
                public void onEvents(Map<Path, WatchEvent.Kind<?>> events) throws IOException {
                    for (Path path : events.keySet()) {
                        LOG.info("Files in path [{}] have changed.", path);
                        build();
                    }
                }
            });
        }
    }

    public synchronized void build() throws IOException {
        LOG.info("Starting build.");
        Map<String, List<Link>> resources = buildResources();
        buildTemplates(resources);
        LOG.info("Build finished.");
    }

    @Override
    public void close() throws IOException {
        if (watchEventObserver != null) {
            watchEventObserver.close();
        }
    }

    private Map<String, List<Link>> buildResources() throws IOException {
        Map<String, List<Link>> resources = new LinkedHashMap<>();

        for (Map.Entry<String, ResourceBuilder> entry : resourceBuildersByFileSourceId.entrySet()) {

            String fileSourceId = entry.getKey();
            ResourceBuilder resourceBuilder = entry.getValue();
            FileSource fileSource = fileSourcesById.get(fileSourceId);

            if (LOG.isDebugEnabled()) {
                LOG.debug(
                        "Building resources for fileSource with id [{}]:\n{}",
                        fileSourceId,
                        join("\n", fileSource.getPaths()));
            }

            List<Link> links = resourceBuilder.build(fileSource);

            if (LOG.isDebugEnabled()) {
                LOG.debug(
                        "Links to resources for fileSource with id [{}]:\n{}",
                        fileSourceId,
                        join("\n", links));
            }
            resources.put(fileSourceId, links);
        }
        return resources;
    }

    private void buildTemplates(Map<String, List<Link>> resources) throws IOException {
        for (Map.Entry<String, TemplateBuilder> entry : templateBuildersByFileSourceId.entrySet()) {
            String fileSourceId = entry.getKey();
            TemplateBuilder templateBuilder = entry.getValue();
            FileSource fileSource = fileSourcesById.get(fileSourceId);

            if (LOG.isDebugEnabled()) {
                LOG.debug(
                        "Building templates for fileSource with id [{}]:\n{}",
                        fileSourceId,
                        join("\n", fileSource.getPaths()));
            }

            Path sourceRoot = fileSource.getSourceRoot();
            Path buildRoot = fileSource.getBuildRoot();
            for (Path path : fileSource.getPaths()) {
                Path templateSource = sourceRoot.resolve(path);
                Path templateTarget = buildRoot.resolve(path);
                Map<String, String> templateContent = buildTemplateContent(templateTarget, resources);
                templateBuilder.build(templateSource, templateTarget, templateContent);
            }
        }
    }

    private Map<String, String> buildTemplateContent(Path templateTarget, Map<String, List<Link>> resources) {
        Map<String, String> templateContent = new LinkedHashMap<>();
        for (Map.Entry<String, List<Link>> entry : resources.entrySet()) {
            List<Link> relativizedLinks = new ArrayList<>();
            for (Link link : entry.getValue()) {
                relativizedLinks.add(link.relativize(templateTarget));
            }
            templateContent.put(entry.getKey(), CollectionUtils.join("\n", relativizedLinks));
        }
        return templateContent;
    }
}
