/**
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 */
package net.amoabeng.util.build;


import java.io.IOException;
import java.nio.charset.Charset;
import java.nio.file.Path;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import static java.nio.file.Files.createDirectories;
import static java.nio.file.Files.readAllLines;
import static java.nio.file.Files.write;
import static java.util.regex.Pattern.compile;
import static java.util.regex.Pattern.quote;
import static net.amoabeng.util.CollectionUtils.join;

/**
 * @author Manuel Amoabeng
 */
class SimpleTemplateBuilder implements TemplateBuilder {

    private final String variableStart;
    private final String variableEnd;
    private final Charset charset;


    public SimpleTemplateBuilder(Charset charset) {
        this(charset, "${", "}");
    }


    public SimpleTemplateBuilder(Charset charset, String variableStart, String variableEnd) {
        this.charset = charset;
        this.variableStart = variableStart;
        this.variableEnd = variableEnd;
    }


    @Override
    public void build(Path source, Path target, Map<String, String> content) throws IOException {
        // read template
        List<String> contentParts = new ArrayList<>();
        List<String> variables = new ArrayList<>();
        int variableStartLength = variableStart.length();
        int variableEndLength = variableEnd.length();
        Pattern pattern = compile(quote(variableStart) + "[\\w\\.]+" + quote(variableEnd));
        String templateContent = join("\n", readAllLines(source, charset));
        Matcher matcher = pattern.matcher(templateContent);
        int currentIndex = 0;
        while (matcher.find()) {
            contentParts.add(templateContent.substring(currentIndex, matcher.start()));
            String group = matcher.group();
            String variable = group.substring(variableStartLength, group.length() - variableEndLength);
            variables.add(variable);
            currentIndex = matcher.end();
        }
        contentParts.add(templateContent.substring(currentIndex));

        // process template
        StringBuilder sb = new StringBuilder();
        Iterator<String> variableIterator = variables.iterator();
        for (String contentPart : contentParts) {
            sb.append(contentPart);
            if (variableIterator.hasNext()) {
                String variable = variableIterator.next();
                String variableContent = content.get(variable);
                if (variableContent != null) {
                    sb.append(variableContent);
                }
            }
        }

        // write template
        createDirectories(target.getParent());
        write(target, Arrays.asList(sb.toString()), charset);
    }
}
