/**
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 */
package net.amoabeng.util.build;

import org.apache.maven.model.Build;
import org.apache.maven.plugin.AbstractMojo;
import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.project.MavenProject;

import java.io.IOException;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.HashMap;
import java.util.Map;

import static java.lang.String.valueOf;
import static net.amoabeng.util.CollectionUtils.merge;
import static net.amoabeng.util.build.BuildUtils.newResourceManager;

/**
 * <p>Mojo for processing web resources using an {@link ResourceManager}.</p>
 * <p>It will look up a custom factory creating a ResourceManager if its class name is provided in
 * <code>settings,producerType</code>. The default factory produces a manager capable of packaging javascript and css files. See
 * {@link DefaultResourceManagerProducer} for details.</p>
 * <p>However, you will need to tell the maven-war-plugin to exclude the files the ResourceManager handles, if they
 * reside within the webapp directory. This is a basic configuration to make things work:</p>
 * <pre>
 *  &lt;plugin&gt;
 *       &lt;groupId&gt;net.amoabeng.utils&lt;/groupId&gt;
 *       &lt;artifactId&gt;build-utils-maven&lt;/artifactId&gt;
 *       &lt;executions&gt;
 *           &lt;execution&gt;
 *               &lt;goals&gt;
 *                   &lt;goal&gt;process-web-resources&lt;/goal&gt;
 *               &lt;/goals&gt;
 *           &lt;/execution&gt;
 *       &lt;/executions&gt;
 *   &lt;/plugin&gt;
 *  &lt;plugin&gt;
 *      &lt;groupId&gt;org.apache.maven.plugins&lt;/groupId&gt;
 *      &lt;artifactId&gt;maven-war-plugin&lt;/artifactId&gt;
 *      &lt;configuration&gt;
 *          &lt;warSourceExcludes&gt;**&#47;*.js,**&#47;*.css,**&#47;*.html&lt;/warSourceExcludes&gt;
 *      &lt;/configuration&gt;
 *  &lt;/plugin&gt;
 * </pre>
 * <p/>
 * <p>In order to be able to "hot-swap" changed resources during development, you may invoke the mojo like this:<br/>
 * mvn build-utils:process-web-resources -DbuildUtils.watch=true</p>
 *
 * @author Manuel Amoabeng
 * @goal process-web-resources
 * @phase process-resources
 */
public class ResourceManagerMojo extends AbstractMojo {
    // TODO test if standalone invocation using watch = true still works

    /**
     * The Maven Project Object
     *
     * @parameter expression="${project}"
     * @required
     */
    private MavenProject project;

    /**
     * The Maven Project Object
     *
     * @parameter expression="${project.build}"
     * @required
     */
    private Build build;

    /**
     * @parameter expression="${project.build.sourceEncoding}"
     */
    private String encoding;

    /**
     * @parameter
     */
    private Map<String, String> settings;

    /**
     * @parameter expression="${buildUtils.watch}"
     */
    private boolean watch;

    /**
     * @parameter expression="${buildUtils.mode}"
     */
    private String mode;

    @Override
    public void execute() throws MojoExecutionException {
        try {
            Path baseDir = project.getBasedir().toPath().normalize().toAbsolutePath();
            Path absoluteBuildRoot = Paths.get(build.getDirectory(), build.getFinalName()).normalize().toAbsolutePath();
            Path buildRoot = baseDir.relativize(absoluteBuildRoot);

            Map<String, String> settings = new HashMap<>();
            settings.put(ResourceManagerProducer.CHARSET, encoding);
            settings.put(ResourceManagerProducer.MODE, mode);
            settings.put(ResourceManagerProducer.BASE_DIR, baseDir.toString());
            settings.put(ResourceManagerProducer.BUILD_ROOT, buildRoot.toString());
            settings.put(ResourceManagerProducer.BUILD_ON_CHANGE, valueOf(watch));

            newResourceManager(merge(settings, this.settings)).build();

        } catch (IOException e) {
            throw new MojoExecutionException("Error executing build-utils:process-web-resources: " + e.getMessage(), e);
        }
    }
}
